/************************************************************************
	main.c

	WFF USB Generic HID Demonstration 3
    usbGenericHidCommunication reference firmware 3_0_0_0
    Copyright (C) 2011 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

#ifndef MAIN_C
#define MAIN_C

// if running on the simulator, the simulator will get stuck in the SPI write code.
// defining debug removes the SPI write code.
//#define DEBUG

// Global includes
// Note: string.h is required for sprintf commands for debug
#include <string.h>

// Local includes
#include "HardwareProfile.h"  
#include "debug.h"

// Microchip Application Library includes
// (expects V2.9a of the USB library from "Microchip Solutions v2011-07-14")
//
// The library location must be set in:
// Project -> Build Options Project -> Directories -> Include search path
// in order for the project to compile.
#include "./USB/usb.h"
#include "./USB/usb_function_hid.h"

#include <delays.h> //delay Library
#include <timers.h>
#include <EEP.H>


struct s {
    unsigned char sl;
    char ch[5];
};

#define table_count 18   //bad Dwayne has 2nd copy of this in patterns_4x4x4.c
extern rom unsigned char *patterns[];
extern rom unsigned int pattern_size[];
extern rom unsigned char pattern_demo_loops[];
extern rom struct s chardata[]; 

// Ensure we have the correct target PIC device family
#if !defined(__18F4550) && !defined(__18F2550) && !defined(__18F2450) && !defined(__18F25K50)
	#error "This firmware only supports either the PIC18F4550 or PIC18F2550 microcontrollers."
#endif

// Define the globals for the USB data in the USB RAM of the PIC18F*550


unsigned char ReceivedDataBuffer[64] @ 0x500;
unsigned char ToSendDataBuffer[64] @ 0x540;


typedef struct {
   unsigned char red;
   unsigned char green;
} strobes;



strobes led_data[128]; // @ 0x200;


//strobes led_data_save[5];


unsigned char side_left_out @ 0x100 =0;
unsigned char side_right_out @ 0x101 =0;
unsigned char wait_timer @ 0x102;
unsigned char strobe @ 0x103 =0;
unsigned char intensity @ 0x104 = 0;
unsigned char idelay @ 0x105 = 0;


USB_HANDLE USBOutHandle = 0;
USB_HANDLE USBInHandle = 0;
BOOL blinkStatusValid = FLAG_TRUE;

unsigned int temp=0;
unsigned int good_ee_pattern;
unsigned int eeprom_msg_size=0;

// PIC18F25K50 configuration for the WFF Generic HID test device
// Internal 16 MHz Osc
#pragma config PLLSEL = PLL3X       // PLL Selection (3x clock multiplier) => 3 x 16 = 48 MHz
#pragma config CFGPLLEN = ON        // PLL Enable Configuration bit (PLL Enabled)
#pragma config CPUDIV = NOCLKDIV    // 1:1 mode (for 48MHz CPU)

// CONFIG1H
#pragma config FOSC = INTOSCIO      // Oscillator Selection (Internal oscillator)
#pragma config PCLKEN = ON          // Primary Oscillator Shutdown (Primary oscillator enabled)
#pragma config FCMEN = OFF          // Fail-Safe Clock Monitor (Fail-Safe Clock Monitor disabled)
#pragma config IESO = OFF           // Internal/External Oscillator Switchover (Oscillator Switchover mode disabled)

// CONFIG2L
#pragma config nPWRTEN = OFF        // Power-up Timer Enable (Power up timer disabled)
#pragma config BOREN = SBORDIS      // Brown-out Reset Enable (BOR enabled in hardware (SBOREN is ignored))
#pragma config BORV = 190           // Brown-out Reset Voltage (BOR set to 1.9V nominal)
#pragma config nLPBOR = OFF         // Low-Power Brown-out Reset (Low-Power Brown-out Reset disabled)

// CONFIG2H
#pragma config WDTEN = OFF         // Watchdog Timer Enable bits (WDT controlled by firmware (SWDTEN enabled))
#pragma config WDTPS = 32768        // Watchdog Timer Postscaler (1:32768)

// CONFIG3H
#pragma config CCP2MX = RC1         // CCP2 MUX bit (CCP2 input/output is multiplexed with RC1)
#pragma config PBADEN = OFF         // PORTB A/D Enable bit (PORTB<5:0> pins are configured as digital I/O on Reset)
#pragma config T3CMX = RB5          // Timer3 Clock Input MUX bit (T3CKI function is on RC0)
#pragma config SDOMX = RB3          // SDO Output MUX bit (SDO function is on RC7)
#pragma config MCLRE = OFF           // Master Clear Reset Pin Enable (MCLR pin enabled; RE3 input disabled)

// CONFIG4L
#pragma config STVREN = ON          // Stack Full/Underflow Reset (Stack full/underflow will cause Reset)
#pragma config LVP = OFF             // Single-Supply ICSP Enable bit (Single-Supply ICSP enabled if MCLRE is also 1)
#pragma config XINST = OFF          // Extended Instruction Set Enable bit (Instruction set extension and Indexed Addressing mode disabled)
//#pragma config ICPRT = OFF                  // ICP

// CONFIG5L
#pragma config CP0 = OFF            // Block 0 Code Protect (Block 0 is not code-protected)
#pragma config CP1 = OFF            // Block 1 Code Protect (Block 1 is not code-protected)
#pragma config CP2 = OFF            // Block 2 Code Protect (Block 2 is not code-protected)
#pragma config CP3 = OFF            // Block 3 Code Protect (Block 3 is not code-protected)

// CONFIG5H
#pragma config CPB = OFF            // Boot Block Code Protect (Boot block is not code-protected)
#pragma config CPD = OFF            // Data EEPROM Code Protect (Data EEPROM is not code-protected)

// CONFIG6L
#pragma config WRT0 = OFF           // Block 0 Write Protect (Block 0 (0800-1FFFh) is not write-protected)
#pragma config WRT1 = OFF           // Block 1 Write Protect (Block 1 (2000-3FFFh) is not write-protected)
#pragma config WRT2 = OFF           // Block 2 Write Protect (Block 2 (04000-5FFFh) is not write-protected)
#pragma config WRT3 = OFF           // Block 3 Write Protect (Block 3 (06000-7FFFh) is not write-protected)

// CONFIG6H
#pragma config WRTC = OFF           // Configuration Registers Write Protect (Configuration registers (300000-3000FFh) are not write-protected)
#pragma config WRTB = OFF           // Boot Block Write Protect (Boot block (0000-7FFh) is not write-protected)
#pragma config WRTD = OFF           // Data EEPROM Write Protect (Data EEPROM is not write-protected)

// CONFIG7L
#pragma config EBTR0 = OFF          // Block 0 Table Read Protect (Block 0 is not protected from table reads executed in other blocks)
#pragma config EBTR1 = OFF          // Block 1 Table Read Protect (Block 1 is not protected from table reads executed in other blocks)
#pragma config EBTR2 = OFF          // Block 2 Table Read Protect (Block 2 is not protected from table reads executed in other blocks)
#pragma config EBTR3 = OFF          // Block 3 Table Read Protect (Block 3 is not protected from table reads executed in other blocks)

// CONFIG7H
#pragma config EBTRB = OFF          // Boot Block Table Read Protect (Boot block is not protected from table reads executed in other blocks)


// Private function prototypes
static void initialisePic(void);
void processUsbCommands(void);
void applicationInit(void);
void USBCBSendResume(void);
void highPriorityISRCode();
void lowPriorityISRCode();
void strobe_LED(unsigned char, unsigned char, unsigned char);
void timer1_isr();
void update_pattern(void);
void next_pattern(void);
void back_pattern(void);
void handle_push_button(void);
void add_and_shift(unsigned char, unsigned char);
unsigned char display_char( unsigned char, unsigned char);
void clear_display(void);
unsigned int brand(void);
unsigned char get_next_pattern_byte(void);
unsigned char pattern_done(void);

// Remap vectors for compatibilty with Microchip USB boot loaders
#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x1000
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x1008
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x1018
#elif defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)	
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x800
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x808
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x818
#else	
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x00
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x08
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x18
#endif

#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER) || defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
	extern void _startup (void);
	#pragma code REMAPPED_RESET_VECTOR = REMAPPED_RESET_VECTOR_ADDRESS
	void _reset (void)
	{
	    _asm goto _startup _endasm
	}
#endif
     
void interrupt high_isr (void)
{

   #if defined(USB_INTERRUPT)
		// Perform USB device tasks
		USBDeviceTasks();
   #endif

// Application specific high-priority ISR code goes here
   if (PIR2bits.TMR3IF){  // Interrupt Check	
      PIR2bits.TMR3IF = 0;

//   TMR3H = 0xF7;
  TMR3H = 0xFC;
   TMR3L = 0;
   
   #asm
//    our data is on page 1
      movlb 0x01
      BANKSEL (_intensity)
      incf BANKMASK(_intensity), 0, 1   // W, BANKED    //    W = intensity + 1
      andlw 0x07                             //    W = W % 8
      movwf BANKMASK(_intensity), 1 // BANKED           //    intensity = W
      bnz   led_strobe1                      //    if (intensity) == 0 then 
      movlw 0x10
      incf BANKMASK(_strobe), 0,1 // W, BANKED         //     W = strobe + 1
      andlw 0x0f                             //    W = W % 16
      movwf BANKMASK(_strobe), 1 // BANKED              //     strobe = W
      bnz   led_strobe1                      //    if (strobe) == 0 then
      incf BANKMASK(_idelay), 1, 1   // F, BANKED       //    W = idelay + 1
      andlw 0x07                             //    W = W % 8
      movwf BANKMASK(_idelay), 1 // BANKED              //    idelay = W
      bnz   led_strobe1                      //    if (idelay) == 0 then
      BANKSEL (_wait_timer)
      clrf  BANKMASK(_wait_timer) // BANKED          //    slow pattern clock
led_strobe1:
     movlw 0x0E
     BANKSEL (_strobe)
     andwf BANKMASK(_strobe), 0,1 // W, BANKED         //     W = strobe & 0b00001110
     mullw 0x08                             //     PROD[HL] = W * 8;

//     FRS0L (pointer0 low) = led_data low + PRODL (multiply low)
      movlw BANKMASK(_led_data) // the "low" is not working!
      addwf PRODL, 0,0 // W, ACCESS
      movwf FSR0L, 0 // ACCESS

//    if strobe odd, add +1 (color bit)
      BANKSEL (_strobe)
      btfsc BANKMASK(_strobe),0,1 // 0 bit, BANKED
      incf  FSR0L, 1,0 // File, ACCESS

//     FRS0H (pointer0 hi) + led_data hi + PRODH (multiply low) + Carry
//     movlw high led_data
      movlw 0x02   // the "high" is not working!, led_data is at 0x0200
      addwfc PRODH, 0,0 // W, ACCESS
      movwf FSR0H, 0 // ACCESS

      clrf  BANKMASK(_side_left_out) // BANKED
//     do this 32 times, skipping every other entry.
//     contents of pointer0 -> W, pointer0++
//     compair intensity with W, skip greater then 
//     set bit or skip
POSTINC0 equ 0FEEh   // DDF because microchip forgot to.
      incf  BANKMASK(_intensity), 0, 1 // W BANKED
      cpfslt POSTINC0  // ACCESS
      bsf   BANKMASK(_side_left_out),0, 1 // BANKED
      cpfslt POSTINC0  // ACCESS
      nop
      cpfslt POSTINC0  // ACCESS
      bsf   BANKMASK(_side_left_out),1, 1 // BANKED
      cpfslt POSTINC0  // ACCESS
      nop
      cpfslt POSTINC0  // ACCESS
      bsf   BANKMASK(_side_left_out),2, 1 // BANKED
      cpfslt POSTINC0  // ACCESS
      nop
      cpfslt POSTINC0  // ACCESS
      bsf   BANKMASK(_side_left_out),3, 1 // BANKED
      cpfslt POSTINC0  // ACCESS
      nop
      cpfslt POSTINC0  // ACCESS
      bsf   BANKMASK(_side_left_out),4, 1 // BANKED
      cpfslt POSTINC0  // ACCESS
      nop
      cpfslt POSTINC0  // ACCESS
      bsf   BANKMASK(_side_left_out),5, 1 // BANKED
      cpfslt POSTINC0  // ACCESS
      nop
      cpfslt POSTINC0  // ACCESS
      bsf   BANKMASK(_side_left_out),6, 1 // BANKED
      cpfslt POSTINC0  // ACCESS
      nop
      cpfslt POSTINC0  // ACCESS
      bsf   BANKMASK(_side_left_out),7, 1 // BANKED
      cpfslt POSTINC0  // ACCESS
      nop

      movwf BANKMASK(_side_right_out) , 1  // BANKED temp store W

      movlw 0x70 // add 112 for 2nd LED bank
      addwf FSR0L, 1,0 // file, ACCESS

      movf BANKMASK(_side_right_out) , 0, 1  // W, BANKED restore W to intestify
      clrf  BANKMASK(_side_right_out) // BANKED

      cpfslt POSTINC0  // ACCESS
      bsf   BANKMASK(_side_right_out),0, 1 // BANKED
      cpfslt POSTINC0  // ACCESS
      nop
      cpfslt POSTINC0  // ACCESS
      bsf   BANKMASK(_side_right_out),1, 1 // BANKED
      cpfslt POSTINC0  // ACCESS
      nop
      cpfslt POSTINC0  // ACCESS
      bsf   BANKMASK(_side_right_out),2, 1 // BANKED
      cpfslt POSTINC0  // ACCESS
      nop
      cpfslt POSTINC0  // ACCESS
      bsf   BANKMASK(_side_right_out),3, 1 // BANKED
      cpfslt POSTINC0  // ACCESS
      nop
      cpfslt POSTINC0  // ACCESS
      bsf   BANKMASK(_side_right_out),4, 1 // BANKED
      cpfslt POSTINC0  // ACCESS
      nop
      cpfslt POSTINC0  // ACCESS
      bsf   BANKMASK(_side_right_out),5, 1 // BANKED
      cpfslt POSTINC0  // ACCESS
      nop
      cpfslt POSTINC0  // ACCESS
      bsf   BANKMASK(_side_right_out),6, 1 // BANKED
      cpfslt POSTINC0  // ACCESS
      nop
      cpfslt POSTINC0  // ACCESS
      bsf   BANKMASK(_side_right_out),7, 1 // BANKED
   #endasm

      strobe_LED(side_left_out, side_right_out, strobe );
   }

}

// Low-priority ISR handling function

void interrupt low_priority low_isr(void)
{
	// Application specific low-priority ISR code goes here
    wait_timer = FLAG_FALSE;
}

// String for creating debug messages
char debugString[80];

BOOL run = FLAG_TRUE;

BOOL demo_mode = FLAG_TRUE;

unsigned char p_intensity=0,table_type=0,cycle_count =0,pattern_speed=0;
char p_table=0;
unsigned int p_count=0;
BOOL p_up_down=FLAG_FALSE;

unsigned char demo_loops;
unsigned char old_button = 0;


// Main program entry point
void main(void)
{   
    unsigned char delay=0;

    srand(0x3421);

	// Initialise and configure the PIC ready to go
    initialisePic();

    // Show that we are up and running
//    mStatusLED0_on();

	// If we are running in interrupt mode attempt to attach the USB device
    #if defined(USB_INTERRUPT)
        USBDeviceAttach();
    #endif
	
	// Initialise the debug log functions
    debugInitialise();
	
 	sprintf(debugString, "2DKits: 8x16 cube");
	debugOut(debugString);
	
 	sprintf(debugString, "USB Device Initialised.");
 	debugOut(debugString);

    if (!sw1) {   // push button on powerup, clear pattern in eeprom.
         Busy_eep();
         Write_b_eep(0x01,0xFF );
         sprintf(debugString, "clear msg in eeprom");
 	     debugOut(debugString);
    }
	
    demo_mode = Read_b_eep(0x00);
    good_ee_pattern = Read_b_eep(0x01);
    eeprom_msg_size = Read_b_eep(0xff);

    demo_loops = pattern_demo_loops[p_table]; // set 1st table delay


	// Main processing loop
    while(1)
    {
        #if defined(USB_POLLING)
			// If we are in polling mode the USB device tasks must be processed here
			// (otherwise the interrupt is performing this task)
	        USBDeviceTasks();
        #endif
    	
    	// Process USB Commands
        processUsbCommands();  

        // Note: Other application specific actions can be placed here
         if ((wait_timer == 0) && (run == TRUE)) { 
            wait_timer = 1;
            delay++;
#ifndef DEBUG
            if ((!sw0) || (!sw1) || old_button) {  
                handle_push_button();
            } else {
#endif
               if (delay > pattern_speed) {
                  delay = 0; 
                  update_pattern();
               }
#ifndef DEBUG
            }
#endif
         }
    } // while   
}

unsigned char fader_cycle=0;
// unsigned char button_delay=0

#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))

unsigned int up_down( unsigned char change, unsigned char data) {

  if (change) {
      return(MIN(8,data+1));
  } else {
      return(MAX(0,(signed char) data-1));
  }
}

void update_pattern() {

   unsigned char i, j, temp_in, red_in, green_in, red_out=1, green_out=0;
   static unsigned char c_red=1, c_green=1;  
   static unsigned char temp_red[8],temp_green[8],char_in,step;
   unsigned char led_row;

// pull extra info bytes off the front of the pattern;

   if (p_count == 0) {
      if ((p_table == 0) && (good_ee_pattern == 3)) {
          get_next_pattern_byte();  //skip over demo flag
      }
      table_type = get_next_pattern_byte();
      pattern_speed = get_next_pattern_byte();
      step = 0;
      red_out=1;
   }

   switch (table_type) {
   case 1:
      for (i=0;i<16;i++) {
         temp_in = get_next_pattern_byte();
         led_row = i*8;
         led_data[led_row+0].red = (temp_in & 0x80) ? (0x07) : (0x00);
         led_data[led_row+1].red = (temp_in & 0x40) ? (0x07) : (0x00);
         led_data[led_row+2].red = (temp_in & 0x20) ? (0x07) : (0x00);
         led_data[led_row+3].red = (temp_in & 0x10) ? (0x07) : (0x00);
         led_data[led_row+4].red = (temp_in & 0x08) ? (0x07) : (0x00);
         led_data[led_row+5].red = (temp_in & 0x04) ? (0x07) : (0x00);
         led_data[led_row+6].red = (temp_in & 0x02) ? (0x07) : (0x00);
         led_data[led_row+7].red = (temp_in & 0x01) ? (0x07) : (0x00);

         temp_in = get_next_pattern_byte();
         led_data[led_row+0].green = (temp_in & 0x80) ? (0x07) : (0x00);
         led_data[led_row+1].green = (temp_in & 0x40) ? (0x07) : (0x00);
         led_data[led_row+2].green = (temp_in & 0x20) ? (0x07) : (0x00);
         led_data[led_row+3].green = (temp_in & 0x10) ? (0x07) : (0x00);
         led_data[led_row+4].green = (temp_in & 0x08) ? (0x07) : (0x00);
         led_data[led_row+5].green = (temp_in & 0x04) ? (0x07) : (0x00);
         led_data[led_row+6].green = (temp_in & 0x02) ? (0x07) : (0x00);
         led_data[led_row+7].green = (temp_in & 0x01) ? (0x07) : (0x00);
      }
      break;
   case 2:

     red_in   = get_next_pattern_byte();
     green_in = get_next_pattern_byte();
     add_and_shift( red_in, green_in );

     break;
   case 3:

     if (step==0) {
        char_in = get_next_pattern_byte();
        while (char_in < 20) {
           c_green = (char_in & 0x01) ? 0x01 : 0x00;
           c_red =  (char_in & 0x02) ? 0x01 : 0x00;
           char_in = get_next_pattern_byte();
        }
        char_in = char_in - 0x20;
        add_and_shift( 0x00, 0x00 );
        step = 1;
     } else {
        add_and_shift( chardata[char_in].ch[step-1] * c_red , chardata[char_in].ch[step-1] * c_green );
        if (step>=chardata[char_in].sl) { step=0; }
        else { step++; }
     }

     break;
   case 4:
      cycle_count = get_next_pattern_byte();
      for (i=0;i<16;i++) {
         temp_in = brand();
         led_row = i*8;
         led_data[led_row+0].red = (temp_in & 0x20) ? (0x07) : (0x00);
         led_data[led_row+1].red = (temp_in & 0x40) ? (0x07) : (0x00);
         led_data[led_row+2].red = (temp_in & 0x80) ? (0x07) : (0x00);
         led_data[led_row+3].red = (temp_in & 0x10) ? (0x07) : (0x00);
         led_data[led_row+4].red = (temp_in & 0x04) ? (0x07) : (0x00);
         led_data[led_row+5].red = (temp_in & 0x01) ? (0x07) : (0x00);
         led_data[led_row+6].red = (temp_in & 0x02) ? (0x07) : (0x00);
         led_data[led_row+7].red = (temp_in & 0x08) ? (0x07) : (0x00);

         temp_in = brand();
         led_data[led_row+0].green = (temp_in & 0x40) ? (0x07) : (0x00);
         led_data[led_row+1].green = (temp_in & 0x20) ? (0x07) : (0x00);
         led_data[led_row+2].green = (temp_in & 0x10) ? (0x07) : (0x00);
         led_data[led_row+3].green = (temp_in & 0x80) ? (0x07) : (0x00);
         led_data[led_row+4].green = (temp_in & 0x01) ? (0x07) : (0x00);
         led_data[led_row+5].green = (temp_in & 0x02) ? (0x07) : (0x00);
         led_data[led_row+6].green = (temp_in & 0x04) ? (0x07) : (0x00);
         led_data[led_row+7].green = (temp_in & 0x08) ? (0x07) : (0x00);
         temp_in = brand(); //make it odd calls
      }
      break;

case 5:
      break;      
   }

   if (pattern_done()) {
      p_count = 0;
      demo_loops = demo_loops - 1;
      if ((demo_loops == 0) & (demo_mode == TRUE)) {
         next_pattern();
      }
   }
}

unsigned char pattern_done()
{
   if ((p_table == 0) && (good_ee_pattern == 3)) {
      return(p_count >= eeprom_msg_size);
   } else {
      return(p_count >= pattern_size[p_table]);
   }
   
}

unsigned char get_next_pattern_byte( )
{
   if ((p_table == 0) && (good_ee_pattern == 3)) {
      return( Read_b_eep(p_count++));
   } else {
      return(patterns[p_table][p_count++]);
   }
}

unsigned brand()
{
    static unsigned int lfsr = 0xACE1 ;
    unsigned int bita;

    bita  = ((lfsr >> 0) ^ (lfsr >> 2) ^ (lfsr >> 3) ^ (lfsr >> 5) ) & 1;
    lfsr =  (lfsr >> 1) | (bita << 15);

    return lfsr;
}



void handle_push_button() {

#ifdef S4X4
#define DEMO_MODE_LED 8
#else
#define DEMO_MODE_LED 32
#endif

   unsigned char i, drow;
   clear_display();
   if (demo_mode) {
      display_char( 0, 0x44 );
   }
   drow = 6;
   if (p_table > 8) {
      display_char( drow, 0x30+((p_table+1)/10) );
      drow = 10; // this is a bug if we have more then 20 tables. display_char should return posistion
   }
   display_char( drow, 0x30+((p_table+1) % 10) );
  
   old_button++;


#define BUTTON_DELAY 64

#ifndef DEBUG
   if ((!sw0) & (!sw1)) {
      if (old_button > BUTTON_DELAY) {
         demo_mode = !demo_mode;
         Busy_eep();
         Write_b_eep(0x00, demo_mode);
         old_button = 1;
      }
   } else if (!sw1) {
      if (old_button > BUTTON_DELAY) {
         next_pattern();
         old_button = 1;
      }
   } else if (!sw0) {
      if (old_button > BUTTON_DELAY) {
         back_pattern();
         old_button = 1;
      }
   } else {
//      for (i=0;i++;i<4) {
//         led_data[i] = led_data_save[i];  // no button, restore the 4 LEDs state.
//      }
//      led_data[DEMO_MODE_LED] = led_data_save[4];  // no button, restore the 4 LEDs state.
      old_button = 0;
      sprintf(debugString, "pattern %d %d", p_table, demo_mode);
 	  debugOut(debugString);
   }
#endif
}

unsigned char display_char( unsigned char row, unsigned char char_in ) {
     unsigned char step, red_in, green_in, offset;


     char_in = char_in - 0x20;
     step = 0;

     while (step < chardata[char_in].sl) {
        offset = row*8;
        step++;
        row++;
        green_in = 0;
        red_in = chardata[char_in].ch[step-1];

        led_data[offset].red     = (red_in   & 0x80) ? (0x07) : (0x00);
        led_data[offset].green   = (green_in & 0x80) ? (0x07) : (0x00);
        led_data[offset+1].red   = (red_in   & 0x40) ? (0x07) : (0x00);
        led_data[offset+1].green = (green_in & 0x40) ? (0x07) : (0x00);
        led_data[offset+2].red   = (red_in   & 0x20) ? (0x07) : (0x00);
        led_data[offset+2].green = (green_in & 0x20) ? (0x07) : (0x00);
        led_data[offset+3].red   = (red_in   & 0x10) ? (0x07) : (0x00);
        led_data[offset+3].green = (green_in & 0x10) ? (0x07) : (0x00);
        led_data[offset+4].red   = (red_in   & 0x08) ? (0x07) : (0x00);
        led_data[offset+4].green = (green_in & 0x08) ? (0x07) : (0x00);
        led_data[offset+5].red   = (red_in   & 0x04) ? (0x07) : (0x00);
        led_data[offset+5].green = (green_in & 0x04) ? (0x07) : (0x00);
        led_data[offset+6].red   = (red_in   & 0x02) ? (0x07) : (0x00);
        led_data[offset+6].green = (green_in & 0x02) ? (0x07) : (0x00);
        led_data[offset+7].red   = (red_in   & 0x01) ? (0x07) : (0x00);
        led_data[offset+7].green = (green_in & 0x01) ? (0x07) : (0x00);
     }
     return(row);
}

void add_and_shift( unsigned char red_in, unsigned char green_in) {

   unsigned char i;
   for (i=0;i<120;i++) {
      led_data[i].red =   led_data[i+8].red;
      led_data[i].green = led_data[i+8].green;
   }

   led_data[120].red   = (red_in   & 0x80) ? (0x07) : (0x00);
   led_data[120].green = (green_in & 0x80) ? (0x07) : (0x00);
   led_data[121].red   = (red_in   & 0x40) ? (0x07) : (0x00);
   led_data[121].green = (green_in & 0x40) ? (0x07) : (0x00);
   led_data[122].red   = (red_in   & 0x20) ? (0x07) : (0x00);
   led_data[122].green = (green_in & 0x20) ? (0x07) : (0x00);
   led_data[123].red   = (red_in   & 0x10) ? (0x07) : (0x00);
   led_data[123].green = (green_in & 0x10) ? (0x07) : (0x00);
   led_data[124].red   = (red_in   & 0x08) ? (0x07) : (0x00);
   led_data[124].green = (green_in & 0x08) ? (0x07) : (0x00);
   led_data[125].red   = (red_in   & 0x04) ? (0x07) : (0x00);
   led_data[125].green = (green_in & 0x04) ? (0x07) : (0x00);
   led_data[126].red   = (red_in   & 0x02) ? (0x07) : (0x00);
   led_data[126].green = (green_in & 0x02) ? (0x07) : (0x00);
   led_data[127].red   = (red_in   & 0x01) ? (0x07) : (0x00);
   led_data[127].green = (green_in & 0x01) ? (0x07) : (0x00);
}

void next_pattern(void) {
      p_table = (p_table >= table_count) ? (0):(p_table+1);
      p_count = 0;
      demo_loops = pattern_demo_loops[p_table];
      sprintf(debugString, "Up to pattern %d %d", p_table, demo_loops);
 	  debugOut(debugString);
}

void back_pattern(void) {
     p_table = (p_table < 1) ? (table_count):(p_table-1);
     p_count = 0;
     demo_loops = pattern_demo_loops[p_table];
     sprintf(debugString, "Down to pattern %d %d", p_table, demo_loops);
 	 debugOut(debugString);
}

extern volatile unsigned char           WPUB               @ 0xF85;  //DDF Because microchip forgot
// Initialise the PIC
static void initialisePic(void)
{
    unsigned char dummySPIdata;
   unsigned char i;

    // PIC port set up --------------------------------------------------------


    // Fix the settings needed for USB
    ACTCONbits.ACTEN = 0; // ACT module is enabled, updates to OSCTUNE are exclusive to the ACT module.
    ACTCONbits.ACTSRC = 1; //Active Clock Tuning Source Selection bit
    ACTCONbits.ACTEN = 1; // ACT module is enabled, updates to OSCTUNE are exclusive to the ACT module.
    ACTCONbits.ACTUD = 0; // Updates to the OSCTUNE register from ACT module are enabled.
    OSCCONbits.IRCF = 7; // HFINTOSC  (16 MHz)


	// Default all pins to digital
    ADCON1 = 0x00;                                                                                                                                                                                                                                                                                             

	// Configure ports as inputs (1) or outputs(0)
	TRISA = 0b00000000;
	TRISB = 0b00000000;
	TRISC = 0b00001000;  //Push Button


	// Clear all ports
	PORTA = 0b00000000;
	PORTB = 0b00000000;
	PORTC = 0b00000000;


	// If you have a VBUS sense pin (for self-powered devices when you
	// want to detect if the USB host is connected) you have to specify
	// your input pin in Ha:rdwareProfile.h
    #if defined(USE_USB_BUS_SENSE_IO)
    	tris_usb_bus_sense = INPUT_PIN;
    #endif
    
    // In the case of a device which can be both self-powered and bus-powered
    // the device must respond correctly to a GetStatus (device) request and
    // tell the host how it is currently powered.
    //
    // To do this you must device a pin which is high when self powered and low
    // when bus powered and define this in HardwareProfile.h
    #if defined(USE_SELF_POWER_SENSE_IO)
    	tris_self_power = INPUT_PIN;
    #endif

    // Application specific initialisation
    applicationInit();
    
    // Initialise the USB device
    USBDeviceInit();

    // setup the LED drivers and SPI buss
 #define LED_col0 		    PORTCbits.RC0
 #define LED_col0_dir		TRISCbits.TRISC0
 #define LED_col1 		    PORTCbits.RC2
 #define LED_col1_dir		TRISCbits.TRISC2
 #define LED_col2 		    PORTCbits.RC6
 #define LED_col2_dir		TRISCbits.TRISC6
 #define LED_col3 		    PORTCbits.RC7
 #define LED_col3_dir		TRISCbits.TRISC7


 #define LED_R_row0 		PORTAbits.RA0
 #define LED_R_row0_dir		TRISAbits.TRISA0
 #define LED_R_row1 		PORTAbits.RA1
 #define LED_R_row1_dir		TRISAbits.TRISA1
 #define LED_R_row2 		PORTAbits.RA2
 #define LED_R_row2_dir		TRISAbits.TRISA2
 #define LED_R_row3 		PORTAbits.RA3
 #define LED_R_row3_dir		TRISAbits.TRISA3
 #define LED_R_row4 		PORTAbits.RA4
 #define LED_R_row4_dir		TRISAbits.TRISA4
 #define LED_R_row5 		PORTAbits.RA5
 #define LED_R_row5_dir		TRISAbits.TRISA5
 #define LED_R_row6 		PORTAbits.RA6
 #define LED_R_row6_dir		TRISAbits.TRISA6
 #define LED_R_row7 		PORTAbits.RA7
 #define LED_R_row7_dir		TRISAbits.TRISA7

 #define LED_G_row0 		PORTBbits.RB0
 #define LED_G_row0_dir		TRISBbits.TRISB0
 #define LED_G_row1 		PORTBbits.RB1
 #define LED_G_row1_dir		TRISBbits.TRISB1
 #define LED_G_row2 		PORTBbits.RB2
 #define LED_G_row2_dir		TRISBbits.TRISB2
 #define LED_G_row3 		PORTBbits.RB3
 #define LED_G_row3_dir		TRISBbits.TRISB3
 #define LED_G_row4 		PORTBbits.RB4
 #define LED_G_row4_dir		TRISBbits.TRISB4
 #define LED_G_row5 		PORTBbits.RB5
 #define LED_G_row5_dir		TRISBbits.TRISB5
 #define LED_G_row6 		PORTBbits.RB6
 #define LED_G_row6_dir		TRISBbits.TRISB6
 #define LED_G_row7 		PORTBbits.RB7
 #define LED_G_row7_dir		TRISBbits.TRISB7

    LED_R_row0         =0;   //off
    LED_R_row1         =0;   //off
    LED_R_row2         =0;   //off
    LED_R_row3         =0;   //off
    LED_R_row4         =0;   //off
    LED_R_row5         =0;   //off
    LED_R_row6         =0;   //off
    LED_R_row7         =0;   //off

    LED_G_row0         =0;   //off
    LED_G_row1         =0;   //off
    LED_G_row2         =0;   //off
    LED_G_row3         =0;   //off
    LED_G_row4         =0;   //off
    LED_G_row5         =0;   //off
    LED_G_row6         =0;   //off
    LED_G_row7         =0;   //off

    LED_R_row0_dir     =0;   //output
    LED_R_row1_dir     =0;   //output
    LED_R_row2_dir     =0;   //output
    LED_R_row3_dir     =0;   //output
    LED_R_row4_dir     =0;   //output
    LED_R_row5_dir     =0;   //output
    LED_R_row6_dir     =0;   //output
    LED_R_row7_dir     =0;   //output

    LED_G_row0_dir     =0;   //output
    LED_G_row1_dir     =0;   //output
    LED_G_row2_dir     =0;   //output
    LED_G_row3_dir     =0;   //output
    LED_G_row4_dir     =0;   //output
    LED_G_row5_dir     =0;   //output
    LED_G_row6_dir     =0;   //output
    LED_G_row7_dir     =0;   //output

    LED_col0         =0;   //off
    LED_col1         =0;   //off
    LED_col2         =0;   //off
    LED_col3         =0;   //off

    LED_col0_dir     =0;   //output
    LED_col1_dir     =0;   //output
    LED_col2_dir     =0;   //output
    LED_col3_dir     =0;   //output

//  OpenTimer3
    T3CON = 0b00000111;
    T3GCON = 0b00000000;
   

//  OpenTimer3 defs are wrong on MPLAB C18 v3.45
//    OpenTimer3(
//                 ++-------  TMR3CS Timer3 Clock Source Select bits  (00 - FOSC/4)
//                 ||++-----  T3CKPS<1:0>: Timer3 Input Clock Prescale Select bits (00 - 1:1)
//                 ||||+----  SOSCEN: Secondary Oscillator Enable Control bit (x)
//                 |||||+---  T3SYNC: Timer3 External Clock Input Synchronization Control bit (1 - not sync)
//                 ||||||+--  RD16: 16-Bit Read/Write Mode Enable bit (1 - enable)
//                 |||||||+-  TMR3ON: Timer3 On bit (1 - enable)
//                 ||||||||
//                 0b00000111,                 // Interrupt enabled

//                 +--------  TMR3GE: Timer3 Gate Enable bit (0 - disable)
//                 |+-------  T3GPOL: Timer3 Gate Polarity bit (x)
//                 ||+------  T3GTM: Timer3 Gate Toggle Mode bit (x)
//                 |||+-----  T3GSPM: Timer3 Gate Single-Pulse Mode bit (x)
//                 ||||+----  T3GGO/DONE: Timer3 Gate Single-Pulse Acquisition Status bit (x)
//                 |||||+---  T3GVAL: Timer3 Gate Current State bit 
//                 ||||||++-  T3GSS<1:0>: Timer3 Gate Source Select bits  (xx)
//                 ||||||||
//                 0b00000000                   //set timer1 as 16 bit registers

//              );

	RCONbits.IPEN=1;     //Enable priority levels on interrupts
	RCONbits.SBOREN=0;   //Disable BOR

    IPR2bits.TMR3IP = 1; //Timer3 interrupt priority hi 
    T3CONbits.TMR3ON = 1;	/* Enable the timer3 */
    PIE2bits.TMR3IE = 1;	/* Enable timer3 interrupt */	
    INTCONbits.GIEH = 1; //enable hi pri interrupts

    INTCON2bits.RBPU = 0;
    WPUB = 0;                // all Port B pull-ups disabled 
//    TRSEbits.WPU3E = 1;      // enable Port E pull-up
//    ANDELE = 0;              // disable port E analog

   clear_display();
}

void clear_display() {
   unsigned char i;

   for (i=0;i<128;i++) {
      led_data[i].red = 0X00;
      led_data[i].green = 0X00;
   }
}

void strobe_LED(unsigned char red, unsigned char green, unsigned char col )
{
static unsigned char old_col = 0;

//  all col off 
if (old_col!=col) {
    PORTB = 0;
    PORTA = 0;

    LATCbits.LATC0= (col & 1)? (1):(0);
    LATCbits.LATC2= (col & 2)? (1):(0);
    LATCbits.LATC6= (col & 4)? (1):(0);
    LATCbits.LATC7= (col & 8)? (1):(0);

    PORTB = red;
    PORTA = green;
  
    old_col = col;
    }
}

// Application specific device initialisation
void applicationInit(void)
{
	// Initialise the status LEDs
//	mInitStatusLeds();
	
	// Initialise the switch
	mInitAllSwitches();
	
    // Initialize the variable holding the USB handle for the last transmission
    USBOutHandle = 0;
    USBInHandle = 0;
}

// Process USB commands
void processUsbCommands(void)
{   
    unsigned char temp,i,done;
    // Check if we are in the configured state; otherwise just return
    if((USBDeviceState < CONFIGURED_STATE) || (USBSuspendControl == 1))
    {
	    // We are not configured
	    return;
	}

	// Check if data was received from the host.
    if(!HIDRxHandleBusy(USBOutHandle))
    {   
		// Command mode    
        switch(ReceivedDataBuffer[0])
		{
			case 0x10:	// Debug information request from host
				// Copy any waiting debug text to the send data buffer
				copyDebugToSendBuffer((BYTE*)&ToSendDataBuffer[0]);
			        
		        // Transmit the response to the host
                if(!HIDTxHandleBusy(USBInHandle))
				{
					USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
				}
				break;
				
			// Place application specific commands here:
			
            case 0x80:  // Toggle the LED
            	sprintf(debugString, "Toggle LED %d", demo_loops);
				debugOut(debugString);

				// Toggle the LED0
//				mStatusLED0_Toggle();

                run = !run;

                done = Read_b_eep(0xff);
                for (i=4;i<done;i++) {
                    debugString[i-4] = Read_b_eep(i);
                    if ((debugString[i-4] > 0) && (debugString[i-4] < 20)) {
                       debugString[i-4]=0x23;
                    }
                }
                debugOut(debugString);
            	break;
            	
            case 0x81:  // Read the push switch status
				ToSendDataBuffer[0] = sw0 + (sw1<<1);

				// Transmit the response to the host
                if(!HIDTxHandleBusy(USBInHandle))
				{
					USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
				}
            	break;
            	
            case 0x82:  // Read the LED status
				// Get the LED state and put it in the send buffer
				ToSendDataBuffer[0] = run;
				
				// Transmit the response to the host
                if(!HIDTxHandleBusy(USBInHandle))
				{
					USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
				}
            	break;

            case 0x83:  // RGB LED set command
				// process an RBG matrix command
		        led_data[ReceivedDataBuffer[4]].red = ReceivedDataBuffer[1];
		        led_data[ReceivedDataBuffer[4]].green = ReceivedDataBuffer[2];
            	break;

            case 0x84:  // pattern change command
				// process an RBG matrix command
		        if (ReceivedDataBuffer[1] == 0) {
                   next_pattern();
                } else if (ReceivedDataBuffer[1] == 1) {
                   back_pattern();
                }
            	break;
            case 0x85:  // eeprom pattern
				// process an RBG matrix command
		        sprintf(debugString, "Got a eeprom pattern >%s<",&ReceivedDataBuffer[4]); //0x20
	            debugOut(debugString);
                for (i=1;ReceivedDataBuffer[i]!=0x00;i++) {
                   Busy_eep();
                   Write_b_eep(i,ReceivedDataBuffer[i] );
                } 
                Busy_eep();
                Write_b_eep(i,0x00 );
                eeprom_msg_size = i+1;
                Busy_eep();
                Write_b_eep(0xff,eeprom_msg_size );
                sprintf(debugString, "saved eeprom");
	            debugOut(debugString);
                good_ee_pattern = Read_b_eep(0x01); 
                if (p_table == 0) {
                   p_count = 0;    // if running pattern 0, restart it.
                }
            	break;
            default:	// Unknown command received
           		break;
		}
        // Re-arm the OUT endpoint for the next packet
        USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
  	}
}

// USB Callback handling routines -----------------------------------------------------------

// Call back that is invoked when a USB suspend is detected
void USBCBSuspend(void)
{
}

// This call back is invoked when a wakeup from USB suspend is detected.
void USBCBWakeFromSuspend(void)
{
}

// The USB host sends out a SOF packet to full-speed devices every 1 ms.
void USBCB_SOF_Handler(void)
{
    // No need to clear UIRbits.SOFIF to 0 here. Callback caller is already doing that.
}

// The purpose of this callback is mainly for debugging during development.
// Check UEIR to see which error causes the interrupt.
void USBCBErrorHandler(void)
{
    // No need to clear UEIR to 0 here.
    // Callback caller is already doing that.
}

// Check other requests callback
void USBCBCheckOtherReq(void)
{
    USBCheckHIDRequest();
}

// Callback function is called when a SETUP, bRequest: SET_DESCRIPTOR request arrives.
void USBCBStdSetDscHandler(void)
{
    // You must claim session ownership if supporting this request
}

//This function is called when the device becomes initialized
void USBCBInitEP(void)
{
    // Enable the HID endpoint
    USBEnableEndpoint(HID_EP,USB_IN_ENABLED|USB_OUT_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
    
    // Re-arm the OUT endpoint for the next packet
    USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
}

// Send resume call-back
void USBCBSendResume(void)
{
    static WORD delay_count;
    
    // Verify that the host has armed us to perform remote wakeup.
    if(USBGetRemoteWakeupStatus() == FLAG_TRUE) 
    {
        // Verify that the USB bus is suspended (before we send remote wakeup signalling).
        if(USBIsBusSuspended() == FLAG_TRUE)
        {
            USBMaskInterrupts();
            
            // Bring the clock speed up to normal running state
            USBCBWakeFromSuspend();
            USBSuspendControl = 0; 
            USBBusIsSuspended = FLAG_FALSE;

            // Section 7.1.7.7 of the USB 2.0 specifications indicates a USB
            // device must continuously see 5ms+ of idle on the bus, before it sends
            // remote wakeup signalling.  One way to be certain that this parameter
            // gets met, is to add a 2ms+ blocking delay here (2ms plus at 
            // least 3ms from bus idle to USBIsBusSuspended() == FLAG_TRUE, yeilds
            // 5ms+ total delay since start of idle).
            delay_count = 3600U;        
            do
            {
                delay_count--;
            } while(delay_count);
            
            // Start RESUME signaling for 1-13 ms
            USBResumeControl = 1;
            delay_count = 1800U;
            do
            {
                delay_count--;
            } while(delay_count);
            USBResumeControl = 0;

            USBUnmaskInterrupts();
        }
    }
}

// USB callback function handler
BOOL USER_USB_CALLBACK_EVENT_HANDLER(int event, void *pdata, WORD size)
{
    switch(event)
    {
        case EVENT_TRANSFER:
            // Application callback tasks and functions go here
            break;
        case EVENT_SOF:
            USBCB_SOF_Handler();
            break;
        case EVENT_SUSPEND:
            USBCBSuspend();
            break;
        case EVENT_RESUME:
            USBCBWakeFromSuspend();
            break;
        case EVENT_CONFIGURED: 
            USBCBInitEP();
            break;
        case EVENT_SET_DESCRIPTOR:
            USBCBStdSetDscHandler();
            break;
        case EVENT_EP0_REQUEST:
            USBCBCheckOtherReq();
            break;
        case EVENT_BUS_ERROR:
            USBCBErrorHandler();
            break;
        default:
            break;
    }      
    return FLAG_TRUE; 
}

#endif
