/************************************************************************
	main.c

	WFF USB Generic HID Demonstration 3
    usbGenericHidCommunication reference firmware 3_0_0_0
    Copyright (C) 2011 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

#ifndef MAIN_C
#define MAIN_C

// if running on the simulator, the simulator will get stuck in the SPI write code.
// defining debug removes the SPI write code.
//#define DEBUG

// changes for the 4x4 square
//#define S4X4

// Global includes
// Note: string.h is required for sprintf commands for debug
#include <string.h>

// Local includes
#include "HardwareProfile.h"  
#include "debug.h"

// Microchip Application Library includes
// (expects V2.9a of the USB library from "Microchip Solutions v2011-07-14")
//
// The library location must be set in:
// Project -> Build Options Project -> Directories -> Include search path
// in order for the project to compile.
#include "./USB/usb.h"
#include "./USB/usb_function_hid.h"

#include <delays.h> //delay Library
#include <timers.h>
#include <EEP.H>

// Ensure we have the correct target PIC device family
#if !defined(__18F4550) && !defined(__18F2550) && !defined(__18F2450) && !defined(__18F25K50)
	#error "This firmware only supports either the PIC18F4550 or PIC18F2550 microcontrollers."
#endif

// Define the globals for the USB data in the USB RAM of the PIC18F*550
#pragma udata
#pragma udata USB_VARIABLES=0x500 
unsigned char ReceivedDataBuffer[64];
unsigned char ToSendDataBuffer[64];
#pragma udata

typedef struct {
   unsigned char red;
   unsigned char green;
} strobes;

USB_HANDLE USBOutHandle = 0;
USB_HANDLE USBInHandle = 0;
BOOL blinkStatusValid = FLAG_TRUE;

// PIC18F25K50 configuration for the WFF Generic HID test device
// Internal 16 MHz Osc
#pragma config PLLSEL = PLL3X       // PLL Selection (3x clock multiplier) => 3 x 16 = 48 MHz
#pragma config CFGPLLEN = ON        // PLL Enable Configuration bit (PLL Enabled)
#pragma config CPUDIV = NOCLKDIV    // 1:1 mode (for 48MHz CPU)

// CONFIG1H
#pragma config FOSC = INTOSCIO      // Oscillator Selection (Internal oscillator)
#pragma config PCLKEN = OFF         // Primary Oscillator Shutdown (Primary oscillator disabled)
#pragma config FCMEN = OFF          // Fail-Safe Clock Monitor (Fail-Safe Clock Monitor disabled)
#pragma config IESO = OFF           // Internal/External Oscillator Switchover (Oscillator Switchover mode disabled)

// CONFIG2L
#pragma config nPWRTEN = OFF        // Power-up Timer Enable (Power up timer disabled)
#pragma config BOREN = SBORDIS      // Brown-out Reset Enable (BOR enabled in hardware (SBOREN is ignored))
#pragma config BORV = 190           // Brown-out Reset Voltage (BOR set to 1.9V nominal)
#pragma config nLPBOR = OFF         // Low-Power Brown-out Reset (Low-Power Brown-out Reset disabled)

// CONFIG2H
#pragma config WDTEN = SWON         // Watchdog Timer Enable bits (WDT controlled by firmware (SWDTEN enabled))
#pragma config WDTPS = 32768        // Watchdog Timer Postscaler (1:32768)

// CONFIG3H
#pragma config CCP2MX = RC1         // CCP2 MUX bit (CCP2 input/output is multiplexed with RC1)
#pragma config PBADEN = OFF         // PORTB A/D Enable bit (PORTB<5:0> pins are configured as digital I/O on Reset)
#pragma config T3CMX = RC0          // Timer3 Clock Input MUX bit (T3CKI function is on RC0)
#pragma config SDOMX = RC7          // SDO Output MUX bit (SDO function is on RC7)
#pragma config MCLRE = OFF           // Master Clear Reset Pin Enable (MCLR pin enabled; RE3 input disabled)

// CONFIG4L
#pragma config STVREN = ON          // Stack Full/Underflow Reset (Stack full/underflow will cause Reset)
#pragma config LVP = ON             // Single-Supply ICSP Enable bit (Single-Supply ICSP enabled if MCLRE is also 1)
#pragma config XINST = OFF          // Extended Instruction Set Enable bit (Instruction set extension and Indexed Addressing mode disabled)
//#pragma config ICPRT = OFF                  // ICP

// CONFIG5L
#pragma config CP0 = OFF            // Block 0 Code Protect (Block 0 is not code-protected)
#pragma config CP1 = OFF            // Block 1 Code Protect (Block 1 is not code-protected)
#pragma config CP2 = OFF            // Block 2 Code Protect (Block 2 is not code-protected)
#pragma config CP3 = OFF            // Block 3 Code Protect (Block 3 is not code-protected)

// CONFIG5H
#pragma config CPB = OFF            // Boot Block Code Protect (Boot block is not code-protected)
#pragma config CPD = OFF            // Data EEPROM Code Protect (Data EEPROM is not code-protected)

// CONFIG6L
#pragma config WRT0 = OFF           // Block 0 Write Protect (Block 0 (0800-1FFFh) is not write-protected)
#pragma config WRT1 = OFF           // Block 1 Write Protect (Block 1 (2000-3FFFh) is not write-protected)
#pragma config WRT2 = OFF           // Block 2 Write Protect (Block 2 (04000-5FFFh) is not write-protected)
#pragma config WRT3 = OFF           // Block 3 Write Protect (Block 3 (06000-7FFFh) is not write-protected)

// CONFIG6H
#pragma config WRTC = OFF           // Configuration Registers Write Protect (Configuration registers (300000-3000FFh) are not write-protected)
#pragma config WRTB = OFF           // Boot Block Write Protect (Boot block (0000-7FFh) is not write-protected)
#pragma config WRTD = OFF           // Data EEPROM Write Protect (Data EEPROM is not write-protected)

// CONFIG7L
#pragma config EBTR0 = OFF          // Block 0 Table Read Protect (Block 0 is not protected from table reads executed in other blocks)
#pragma config EBTR1 = OFF          // Block 1 Table Read Protect (Block 1 is not protected from table reads executed in other blocks)
#pragma config EBTR2 = OFF          // Block 2 Table Read Protect (Block 2 is not protected from table reads executed in other blocks)
#pragma config EBTR3 = OFF          // Block 3 Table Read Protect (Block 3 is not protected from table reads executed in other blocks)

// CONFIG7H
#pragma config EBTRB = OFF          // Boot Block Table Read Protect (Boot block is not protected from table reads executed in other blocks)


// Private function prototypes
static void initialisePic(void);
void processUsbCommands(void);
void USBCBSendResume(void);
void highPriorityISRCode();
void lowPriorityISRCode();
void handle_push_button(void);

// Remap vectors for compatibilty with Microchip USB boot loaders
#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x1000
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x1008
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x1018
#elif defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)	
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x800
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x808
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x818
#else	
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x00
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x08
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x18
#endif

#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER) || defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
	extern void _startup (void);
	#pragma code REMAPPED_RESET_VECTOR = REMAPPED_RESET_VECTOR_ADDRESS
	void _reset (void)
	{
	    _asm goto _startup _endasm
	}
#endif

#pragma code REMAPPED_HIGH_INTERRUPT_VECTOR = REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS
void Remapped_High_ISR (void)
{
     _asm goto highPriorityISRCode _endasm
}

#pragma code REMAPPED_LOW_INTERRUPT_VECTOR = REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS
void Remapped_Low_ISR (void)
{
     _asm goto lowPriorityISRCode _endasm
}

#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER) || defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
#pragma code HIGH_INTERRUPT_VECTOR = 0x08
void High_ISR (void)
{
     _asm goto REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS _endasm
}

#pragma code LOW_INTERRUPT_VECTOR = 0x18
void Low_ISR (void)
{
     _asm goto REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS _endasm
}
#endif

#pragma code

// High-priority ISR handling function
#pragma interrupt highPriorityISRCode
void highPriorityISRCode()
{

   #if defined(USB_INTERRUPT)
		// Perform USB device tasks
		USBDeviceTasks();
   #endif

// Application specific high-priority ISR code goes here
}

// Low-priority ISR handling function
#pragma interruptlow lowPriorityISRCode
void lowPriorityISRCode()
{
	// Application specific low-priority ISR code goes here
}

// String for creating debug messages
char debugString[80];

BOOL run = FLAG_TRUE;
BOOL demo_mode = FLAG_TRUE;

// Main program entry point
void main(void)
{   
    char hello[20] = {"2DKits: 8x16 matrix"};
    unsigned char delay=0;

	// Initialise and configure the PIC ready to go
    initialisePic();

	// If we are running in interrupt mode attempt to attach the USB device
    #if defined(USB_INTERRUPT)
        USBDeviceAttach();
    #endif
	
	// Initialise the debug log functions
//    debugInitialise();
	
//	sprintf(debugString, "2DKits: 4x4x4 cube");
//	debugOut(debugString);
//	debugOut(hello);
	
//	sprintf(debugString, "USB Device Initialised.");
//	debugOut(debugString);
	

	// Main processing loop
    while(1)
    {
        #if defined(USB_POLLING)
			// If we are in polling mode the USB device tasks must be processed here
			// (otherwise the interrupt is performing this task)
	        USBDeviceTasks();
        #endif
    	
    	// Process USB Commands
        processUsbCommands();  
    }
}

// Initialise the PIC
static void initialisePic(void)
{
    unsigned char dummySPIdata;
   unsigned char i;

    // PIC port set up --------------------------------------------------------

	// Default all pins to digital
    ADCON1 = 0x0F;                                                                                                                                                                                                                                                                                             

	// Configure ports as inputs (1) or outputs(0)
	TRISA = 0b00000000;
	TRISB = 0b00000000;
	TRISC = 0b00000000;


	// Clear all ports
	PORTA = 0b00000000;
	PORTB = 0b00000000;
	PORTC = 0b00000000;


	// If you have a VBUS sense pin (for self-powered devices when you
	// want to detect if the USB host is connected) you have to specify
	// your input pin in Ha:rdwareProfile.h
    #if defined(USE_USB_BUS_SENSE_IO)
    	tris_usb_bus_sense = INPUT_PIN;
    #endif
    
    // In the case of a device which can be both self-powered and bus-powered
    // the device must respond correctly to a GetStatus (device) request and
    // tell the host how it is currently powered.
    //
    // To do this you must device a pin which is high when self powered and low
    // when bus powered and define this in HardwareProfile.h
    #if defined(USE_SELF_POWER_SENSE_IO)
    	tris_self_power = INPUT_PIN;
    #endif
    
    // Initialise the USB device
    USBDeviceInit();

    // setup the LED drivers and SPI buss
 #define LED_col0 		    PORTCbits.RC0
 #define LED_col0_dir		TRISCbits.TRISC0
 #define LED_col1 		    PORTCbits.RC2
 #define LED_col1_dir		TRISCbits.TRISC2
 #define LED_col2 		    PORTCbits.RC6
 #define LED_col2_dir		TRISCbits.TRISC6
 #define LED_col3 		    PORTCbits.RC7
 #define LED_col3_dir		TRISCbits.TRISC7


 #define LED_R_row0 		PORTAbits.RA0
 #define LED_R_row0_dir		TRISAbits.TRISA0
 #define LED_R_row1 		PORTAbits.RA1
 #define LED_R_row1_dir		TRISAbits.TRISA1
 #define LED_R_row2 		PORTAbits.RA2
 #define LED_R_row2_dir		TRISAbits.TRISA2
 #define LED_R_row3 		PORTAbits.RA3
 #define LED_R_row3_dir		TRISAbits.TRISA3
 #define LED_R_row4 		PORTAbits.RA4
 #define LED_R_row4_dir		TRISAbits.TRISA4
 #define LED_R_row5 		PORTAbits.RA5
 #define LED_R_row5_dir		TRISAbits.TRISA5
 #define LED_R_row6 		PORTAbits.RA6
 #define LED_R_row6_dir		TRISAbits.TRISA6
 #define LED_R_row7 		PORTAbits.RA7
 #define LED_R_row7_dir		TRISAbits.TRISA7

 #define LED_G_row0 		PORTBbits.RB0
 #define LED_G_row0_dir		TRISBbits.TRISB0
 #define LED_G_row1 		PORTBbits.RB1
 #define LED_G_row1_dir		TRISBbits.TRISB1
 #define LED_G_row2 		PORTBbits.RB2
 #define LED_G_row2_dir		TRISBbits.TRISB2
 #define LED_G_row3 		PORTBbits.RB3
 #define LED_G_row3_dir		TRISBbits.TRISB3
 #define LED_G_row4 		PORTBbits.RB4
 #define LED_G_row4_dir		TRISBbits.TRISB4
 #define LED_G_row5 		PORTBbits.RB5
 #define LED_G_row5_dir		TRISBbits.TRISB5
 #define LED_G_row6 		PORTBbits.RB6
 #define LED_G_row6_dir		TRISBbits.TRISB6
 #define LED_G_row7 		PORTBbits.RB7
 #define LED_G_row7_dir		TRISBbits.TRISB7

    LED_R_row0         =0;   //off
    LED_R_row1         =0;   //off
    LED_R_row2         =0;   //off
    LED_R_row3         =0;   //off
    LED_R_row4         =0;   //off
    LED_R_row5         =0;   //off
    LED_R_row6         =0;   //off
    LED_R_row7         =0;   //off

    LED_G_row0         =0;   //off
    LED_G_row1         =0;   //off
    LED_G_row2         =0;   //off
    LED_G_row3         =0;   //off
    LED_G_row4         =0;   //off
    LED_G_row5         =0;   //off
    LED_G_row6         =0;   //off
    LED_G_row7         =0;   //off

    LED_R_row0_dir     =0;   //output
    LED_R_row1_dir     =0;   //output
    LED_R_row2_dir     =0;   //output
    LED_R_row3_dir     =0;   //output
    LED_R_row4_dir     =0;   //output
    LED_R_row5_dir     =0;   //output
    LED_R_row6_dir     =0;   //output
    LED_R_row7_dir     =0;   //output

    LED_G_row0_dir     =0;   //output
    LED_G_row1_dir     =0;   //output
    LED_G_row2_dir     =0;   //output
    LED_G_row3_dir     =0;   //output
    LED_G_row4_dir     =0;   //output
    LED_G_row5_dir     =0;   //output
    LED_G_row6_dir     =0;   //output
    LED_G_row7_dir     =0;   //output

    LED_col0         =0;   //off
    LED_col1         =0;   //off
    LED_col2         =0;   //off
    LED_col3         =0;   //off

    LED_col0_dir     =0;   //output
    LED_col1_dir     =0;   //output
    LED_col2_dir     =0;   //output
    LED_col3_dir     =0;   //output

//  OpenTimer3
   _asm
      movlw 0b00000111  
      movwf T3CON, 0 // ACCESS
      movlw 0b00000000  
      movwf T3GCON, 0 // ACCESS
   _endasm
   

//  OpenTimer3 defs are wrong on MPLAB C18 v3.45
//    OpenTimer3(
//                 ++-------  TMR3CS Timer3 Clock Source Select bits  (00 - FOSC/4)
//                 ||++-----  T3CKPS<1:0>: Timer3 Input Clock Prescale Select bits (00 - 1:1)
//                 ||||+----  SOSCEN: Secondary Oscillator Enable Control bit (x)
//                 |||||+---  T3SYNC: Timer3 External Clock Input Synchronization Control bit (1 - not sync)
//                 ||||||+--  RD16: 16-Bit Read/Write Mode Enable bit (1 - enable)
//                 |||||||+-  TMR3ON: Timer3 On bit (1 - enable)
//                 ||||||||
//                 0b00000111,                 // Interrupt enabled

//                 +--------  TMR3GE: Timer3 Gate Enable bit (0 - disable)
//                 |+-------  T3GPOL: Timer3 Gate Polarity bit (x)
//                 ||+------  T3GTM: Timer3 Gate Toggle Mode bit (x)
//                 |||+-----  T3GSPM: Timer3 Gate Single-Pulse Mode bit (x)
//                 ||||+----  T3GGO/DONE: Timer3 Gate Single-Pulse Acquisition Status bit (x)
//                 |||||+---  T3GVAL: Timer3 Gate Current State bit 
//                 ||||||++-  T3GSS<1:0>: Timer3 Gate Source Select bits  (xx)
//                 ||||||||
//                 0b00000000                   //set timer1 as 16 bit registers

//              );

	RCONbits.IPEN=1;     //Enable priority levels on interrupts
	RCONbits.SBOREN=0;   //Disable BOR

    IPR2bits.TMR3IP = 1; //Timer3 interrupt priority hi 
    T3CONbits.TMR3ON = 1;	/* Enable the timer3 */
    PIE2bits.TMR3IE = 1;	/* Enable timer3 interrupt */	
    INTCONbits.GIEL = 1; //enable low pri interrupts
    UCFGbits.FSEN = 1; //

//  Stuff for USB
//
    ACTCONbits.ACTEN = 0; // ACT module is enabled, updates to OSCTUNE are exclusive to the ACT module.
    ACTCONbits.ACTSRC = 1; //Active Clock Tuning Source Selection bit
    ACTCONbits.ACTEN = 1; // ACT module is enabled, updates to OSCTUNE are exclusive to the ACT module.
    ACTCONbits.ACTUD = 0; // Updates to the OSCTUNE register from ACT module are enabled.
    OSCCONbits.IRCF = 7; // HFINTOSC  (16 MHz)

}

// Process USB commands
void processUsbCommands(void)
{   
    // Check if we are in the configured state; otherwise just return
    if((USBDeviceState < CONFIGURED_STATE) || (USBSuspendControl == 1))
    {
	    // We are not configured
	    return;
	}

	// Check if data was received from the host.
    if(!HIDRxHandleBusy(USBOutHandle))
    {   
		// Command mode    
        switch(ReceivedDataBuffer[0])
		{
			case 0x10:	// Debug information request from host
				// Copy any waiting debug text to the send data buffer
				copyDebugToSendBuffer((BYTE*)&ToSendDataBuffer[0]);
			        
		        // Transmit the response to the host
                if(!HIDTxHandleBusy(USBInHandle))
				{
					USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
				}
				break;
				
			// Place application specific commands here:
			
            case 0x80:  // Toggle the LED
//             	sprintf(debugString, "Toggle LED %d", run);
//				debugOut(debugString);

				// Toggle the LED0
//				mStatusLED0_Toggle();
                run = !run;
            	break;
            	
            case 0x81:  // Read the push switch status
//				ToSendDataBuffer[0] = sw0 + (sw1<<1);

				// Transmit the response to the host
                if(!HIDTxHandleBusy(USBInHandle))
				{
					USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
				}
            	break;
            	
            case 0x82:  // Read the LED status
				// Get the LED state and put it in the send buffer
//				ToSendDataBuffer[0] = mStatusLED0_Get();
				
				// Transmit the response to the host
                if(!HIDTxHandleBusy(USBInHandle))
				{
					USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
				}
            	break;

            default:	// Unknown command received
           		break;
		}
        // Re-arm the OUT endpoint for the next packet
        USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
  	}
}

// USB Callback handling routines -----------------------------------------------------------

// Call back that is invoked when a USB suspend is detected
void USBCBSuspend(void)
{
}

// This call back is invoked when a wakeup from USB suspend is detected.
void USBCBWakeFromSuspend(void)
{
}

// The USB host sends out a SOF packet to full-speed devices every 1 ms.
void USBCB_SOF_Handler(void)
{
    // No need to clear UIRbits.SOFIF to 0 here. Callback caller is already doing that.
}

// The purpose of this callback is mainly for debugging during development.
// Check UEIR to see which error causes the interrupt.
void USBCBErrorHandler(void)
{
    // No need to clear UEIR to 0 here.
    // Callback caller is already doing that.
}

// Check other requests callback
void USBCBCheckOtherReq(void)
{
    USBCheckHIDRequest();
}

// Callback function is called when a SETUP, bRequest: SET_DESCRIPTOR request arrives.
void USBCBStdSetDscHandler(void)
{
    // You must claim session ownership if supporting this request
}

//This function is called when the device becomes initialized
void USBCBInitEP(void)
{
    // Enable the HID endpoint
    USBEnableEndpoint(HID_EP,USB_IN_ENABLED|USB_OUT_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
    
    // Re-arm the OUT endpoint for the next packet
    USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
}

// Send resume call-back
void USBCBSendResume(void)
{
    static WORD delay_count;
    
    // Verify that the host has armed us to perform remote wakeup.
    if(USBGetRemoteWakeupStatus() == FLAG_TRUE) 
    {
        // Verify that the USB bus is suspended (before we send remote wakeup signalling).
        if(USBIsBusSuspended() == FLAG_TRUE)
        {
            USBMaskInterrupts();
            
            // Bring the clock speed up to normal running state
            USBCBWakeFromSuspend();
            USBSuspendControl = 0; 
            USBBusIsSuspended = FLAG_FALSE;

            // Section 7.1.7.7 of the USB 2.0 specifications indicates a USB
            // device must continuously see 5ms+ of idle on the bus, before it sends
            // remote wakeup signalling.  One way to be certain that this parameter
            // gets met, is to add a 2ms+ blocking delay here (2ms plus at 
            // least 3ms from bus idle to USBIsBusSuspended() == FLAG_TRUE, yeilds
            // 5ms+ total delay since start of idle).
            delay_count = 3600U;        
            do
            {
                delay_count--;
            } while(delay_count);
            
            // Start RESUME signaling for 1-13 ms
            USBResumeControl = 1;
            delay_count = 1800U;
            do
            {
                delay_count--;
            } while(delay_count);
            USBResumeControl = 0;

            USBUnmaskInterrupts();
        }
    }
}

// USB callback function handler
BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size)
{
    switch(event)
    {
        case EVENT_TRANSFER:
            // Application callback tasks and functions go here
            break;
        case EVENT_SOF:
            USBCB_SOF_Handler();
            break;
        case EVENT_SUSPEND:
            USBCBSuspend();
            break;
        case EVENT_RESUME:
            USBCBWakeFromSuspend();
            break;
        case EVENT_CONFIGURED: 
            USBCBInitEP();
            break;
        case EVENT_SET_DESCRIPTOR:
            USBCBStdSetDscHandler();
            break;
        case EVENT_EP0_REQUEST:
            USBCBCheckOtherReq();
            break;
        case EVENT_BUS_ERROR:
            USBCBErrorHandler();
            break;
        default:
            break;
    }      
    return FLAG_TRUE; 
}

#endif
